"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Storage = void 0;
const utils_1 = require("@liskhq/lisk-cryptography/dist-node/utils");
const lisk_db_1 = require("@liskhq/lisk-db");
const codec_1 = require("./codec");
const constants_1 = require("./constants");
class Storage {
    constructor(db) {
        this._db = db;
    }
    async getBlockByID(id) {
        return this._db.get(Buffer.concat([constants_1.DB_KEY_BLOCK_ID, id]));
    }
    async getBlockByHeight(height) {
        return this.getBlockByID(await this._db.get(Buffer.concat([constants_1.DB_KEY_BLOCK_HEIGHT, (0, utils_1.intToBuffer)(height, 4)])));
    }
    async getBlocksByHeightBetween(fromHeight, toHeight) {
        const blockIDs = await this._getBlockIDsBetweenHeights(fromHeight, toHeight);
        return Promise.all(blockIDs.map(async (id) => this.getBlockByID(id)));
    }
    async isBlockPersisted(blockID) {
        return this._db.has(Buffer.concat([constants_1.DB_KEY_BLOCK_ID, blockID]));
    }
    async isBlockHeightPersisted(height) {
        return this._db.has(Buffer.concat([constants_1.DB_KEY_BLOCK_HEIGHT, (0, utils_1.intToBuffer)(height, 4)]));
    }
    async saveBlock(id, height, block) {
        const batch = new lisk_db_1.Batch();
        batch.set(Buffer.concat([constants_1.DB_KEY_BLOCK_ID, id]), block);
        batch.set(Buffer.concat([constants_1.DB_KEY_BLOCK_HEIGHT, (0, utils_1.intToBuffer)(height, 4)]), id);
        await this._db.write(batch);
    }
    async getLegacyChainBracketInfo(snapshotBlockID) {
        return this._db.get(Buffer.concat([constants_1.DB_KEY_LEGACY_BRACKET, snapshotBlockID]));
    }
    async setLegacyChainBracketInfo(snapshotBlockID, bracketInfo) {
        await this._db.set(Buffer.concat([constants_1.DB_KEY_LEGACY_BRACKET, snapshotBlockID]), (0, codec_1.encodeLegacyChainBracketInfo)(bracketInfo));
    }
    async _getBlockIDsBetweenHeights(fromHeight, toHeight) {
        const stream = this._db.createReadStream({
            gte: Buffer.concat([constants_1.DB_KEY_BLOCK_HEIGHT, (0, utils_1.intToBuffer)(fromHeight, 4)]),
            lte: Buffer.concat([constants_1.DB_KEY_BLOCK_HEIGHT, (0, utils_1.intToBuffer)(toHeight, 4)]),
            reverse: true,
        });
        return new Promise((resolve, reject) => {
            const ids = [];
            stream
                .on('data', ({ value }) => {
                ids.push(value);
            })
                .on('error', error => {
                reject(error);
            })
                .on('end', () => {
                resolve(ids);
            });
        });
    }
}
exports.Storage = Storage;
//# sourceMappingURL=storage.js.map